import warnings
from unittest import mock

from django.core import checks
from django.test import SimpleTestCase, override_settings

from ambient_toolbox.system_checks.model_relation_conventions import check_model_related_names_for_related_name
from testapp.models import ModelWithoutRelatedNameOnFieldAndMeta


class CheckModelRelationConventionsTestCase(SimpleTestCase):
    def test_check_model_related_names_for_related_name_regular(self):
        # Create expected warning
        relation_warning = checks.Warning(
            "'ModelWithoutRelatedNameOnFieldAndMeta.relation_field' doesn't have a related name set and neither "
            "does the model define a default related name.",
            obj=ModelWithoutRelatedNameOnFieldAndMeta.relation_field.field,
            id="ambient_toolbox.W003",
        )

        # Call system check
        error_list = check_model_related_names_for_related_name()

        # Assert warngins
        self.assertEqual(len(error_list), 1)
        self.assertIn(relation_warning, error_list)

    @override_settings(LOCAL_APPS=["testapp"])
    def test_check_local_apps_are_detected(self):
        # Call system check
        error_list = check_model_related_names_for_related_name()

        # Assert warngins
        self.assertEqual(len(error_list), 1)
        self.assertEqual(error_list[0].id, "ambient_toolbox.W003")

    @override_settings(INSTALLED_APPS="")
    def test_check_no_local_apps_found(self):
        # Call system check
        with mock.patch.object(warnings, "warn") as mocked_warning:
            error_list = check_model_related_names_for_related_name()

        # Assert warngins
        self.assertEqual(len(error_list), 0)
        mocked_warning.assert_called_once_with(
            "No local apps detected. Therefore, no model fields will be checked for missing related names.",
            Warning,
            stacklevel=2,
        )



import warnings

from django.apps import apps
from django.conf import settings
from django.core import checks
from django.db.models import DateField, DateTimeField


def check_model_time_based_fields(*args, **kwargs):
    """
    Checks all model time fields ('DateField', 'DateTimeField') for a "correct" ending in their name.
    Inspired by: https://lukeplant.me.uk/blog/posts/enforcing-conventions-in-django-projects-with-introspection/
    """

    # If local apps are set up separately...
    if getattr(settings, "LOCAL_APPS", None):
        project_apps = [app.split(".")[-1] for app in settings.LOCAL_APPS]
    # Otherwise, we try some magic...
    else:
        project_apps = [
            app.split(".")[-1] for app in settings.INSTALLED_APPS if app.startswith(settings.ROOT_URLCONF.split(".")[0])
        ]

    # If we still don't get any apps, inform the user
    if len(project_apps) == 0:
        warnings.warn(
            "No local apps detected. Therefore, no model fields will be checked for date-time-conventions.",
            Warning,
            stacklevel=2,
        )

    issue_list = []

    # Allowlists
    allowed_datetime_field_endings = getattr(settings, "ALLOWED_MODEL_DATETIME_FIELD_ENDINGS", ["_at"])
    allowed_date_field_endings = getattr(settings, "ALLOWED_MODEL_DATE_FIELD_ENDINGS", ["_date"])

    str_allowed_datetime_endings = ", ".join(allowed_datetime_field_endings)
    str_allowed_date_endings = ", ".join(allowed_date_field_endings)

    # Iterate all registered models...
    for model in apps.get_models():
        # Check if the model is from your project...
        if model._meta.app_label in project_apps:
            # Iterate over all fields...
            for field in model._meta.get_fields():
                # Case: DateTimeField, noqa: ERA001
                if isinstance(field, DateTimeField):
                    # Check field name ending against allowlist
                    if not field.name.lower().endswith(tuple(allowed_datetime_field_endings)):
                        issue_list.append(
                            checks.Warning(
                                f"DateTimeField '{model.__name__}.{field.name}' doesn't end with: "
                                f"{str_allowed_datetime_endings}.",
                                obj=field,
                                id="ambient_toolbox.W001",
                            )
                        )
                # Case: Date field, noqa: ERA001
                elif isinstance(field, DateField):
                    # Check field name ending against allowlist
                    if not field.name.lower().endswith(tuple(allowed_date_field_endings)):
                        issue_list.append(
                            checks.Warning(
                                f"DateField '{model.__name__}.{field.name}' doesn't end with: "
                                f"{str_allowed_date_endings}.",
                                obj=field,
                                id="ambient_toolbox.W002",
                            )
                        )

    return issue_list



from django.test import TestCase

from ambient_toolbox.sentry.helpers import strip_sensitive_data_from_sentry_event


class SentryHelperTest(TestCase):
    def test_strip_sensitive_data_from_sentry_event_regular(self):
        event = {"user": {"email": "mymail@example.com", "ip_address": "127.0.0.1", "username": "my-user"}}

        self.assertIsInstance(strip_sensitive_data_from_sentry_event(event, None), dict)

    def test_strip_sensitive_data_from_sentry_event_missing_key_email(self):
        event = {"user": {"ip_address": "127.0.0.1", "username": "my-user"}}

        self.assertIsInstance(strip_sensitive_data_from_sentry_event(event, None), dict)

    def test_strip_sensitive_data_from_sentry_event_missing_key_ip_address(self):
        event = {"user": {"email": "mymail@example.com", "username": "my-user"}}

        stripped_event = strip_sensitive_data_from_sentry_event(event, None)

        self.assertIsInstance(stripped_event, dict)
        self.assertNotIn("email", stripped_event["user"].keys())
        self.assertNotIn("username", stripped_event["user"].keys())

    def test_strip_sensitive_data_from_sentry_event_missing_key_username(self):
        event = {"user": {"email": "mymail@example.com", "ip_address": "127.0.0.1"}}

        stripped_event = strip_sensitive_data_from_sentry_event(event, None)

        self.assertIsInstance(stripped_event, dict)
        self.assertNotIn("email", stripped_event["user"].keys())
        self.assertNotIn("ip_address", stripped_event["user"].keys())



import calendar
import datetime
from calendar import monthrange
from typing import Optional, Union

import pytz
from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from pytz import UnknownTimeZoneError


class DateHelper:
    """
    Constants to use for django ORMs `__weekday` lookup to avoid usage of integers directly.
    Unfortunately python's calendar weekdays are not equivalent to the database ones.
    """

    ORM_SUNDAY = 1
    ORM_MONDAY = 2
    ORM_TUESDAY = 3
    ORM_WEDNESDAY = 4
    ORM_THURSDAY = 5
    ORM_FRIDAY = 6
    ORM_SATURDAY = 7


def tz_today(str_format=None):
    """
    Returns either a timezone-aware today object or string. Depending on if the `str_format` param is set.
    :param str_format: If set, formats the date object to a string. Pass strftime-compatible value
    :return:
    """
    if settings.USE_TZ:
        date = timezone.now().date()
    else:
        date = datetime.datetime.now().date()

    if str_format:
        return date.strftime(str_format)
    return date


def add_months(
    source_date: Union[datetime.date, datetime.datetime], months: Union[int, float]
) -> Union[datetime.date, datetime.datetime]:
    return source_date + relativedelta(months=months)


def add_days(
    source_date: Union[datetime.date, datetime.datetime], days: int
) -> Union[datetime.date, datetime.datetime]:
    return source_date + datetime.timedelta(days=days)


def add_minutes(source_datetime: datetime.datetime, minutes: int) -> Union[datetime.date, datetime.datetime]:
    return source_datetime + relativedelta(minutes=minutes)


def get_next_month() -> datetime.date:
    return add_months(tz_today(), 1)


def first_day_of_month(source_date: datetime.date) -> datetime.date:
    return datetime.date(source_date.year, source_date.month, 1)


def get_formatted_date_str(source_date: Union[datetime.date, datetime.datetime]) -> str:
    return source_date.strftime("%d.%m.%Y")


def get_time_from_seconds(seconds: int) -> str:
    if seconds < 0:
        raise ValueError(_("Seconds must be positive."))
    hours = seconds // 3600
    minutes = (seconds - (hours * 3600)) // 60
    seconds = seconds - ((hours * 3600) + (minutes * 60))
    return "%02d:%02d:%02d" % (hours, minutes, seconds)


def datetime_format(target_datetime: datetime.datetime, dt_format: str) -> str:
    """
    Uses strftime, but considers timezone (only for datetime objects)
    """
    try:
        dt_format = target_datetime.astimezone(tz=pytz.timezone(settings.TIME_ZONE)).strftime(dt_format)
    except UnknownTimeZoneError:
        dt_format = target_datetime.strftime(dt_format)
    return dt_format


def get_start_and_end_date_from_calendar_week(year: int, calendar_week: int) -> (datetime.date, datetime.date):
    """
    Returns the first and last day of a given calendar week
    """
    start_of_week = datetime.datetime.strptime(f"{year}-{calendar_week}-1", "%Y-%W-%w").astimezone().date()

    if calendar_week == 1:
        # Calculating first_day_of_week for calendar_week == 1 is always tricky, as the first calendar week
        # can actually start in the n-1 year. The above .strptime() can not handle this however, and would always
        # return the first of January.
        # For this case, we consider the fourth of january, as it will _always_ be in the first calendar week
        # (see: https://en.wikipedia.org/wiki/ISO_week_date#First_week)

        fourth_of_january = datetime.date(year=year, month=1, day=4)

        start_of_week = fourth_of_january - relativedelta(days=fourth_of_january.weekday())

    return start_of_week, start_of_week + relativedelta(days=6)


def get_next_calendar_week(compare_date: datetime.date) -> int:
    """
    Returns the next calendar week as an integer
    """
    day_in_one_week = compare_date + datetime.timedelta(days=(7 - compare_date.weekday()))
    return day_in_one_week.isocalendar()[1]


def next_weekday(given_date: datetime.date, weekday: int) -> datetime.date:
    """
    Returns the next date of the given weekday
    For example: next_weekday(d, calendar.MONDAY) would return the next monday starting at date "given_date"
    """
    days_ahead = weekday - given_date.weekday()
    if days_ahead <= 0:
        days_ahead += 7
    return given_date + datetime.timedelta(days_ahead)


def date_month_delta(start_date: datetime.date, end_date: datetime.date) -> float:
    """
    Calculates the number of months lying between two dates.
    So from April 15th to May 1st it's 0.5 months.
    Attention: `end_date` will be excluded in the result (outer border)
    """
    # If `start_date` is greater, this logic doesn't make any sense
    if start_date > end_date:
        raise NotImplementedError("Start date > end date")

    # Calculate date difference between dates
    date_diff = (end_date - start_date).days

    # Iteration variable
    delta = 0
    iter_date = start_date
    # Loop until all days are processed
    while date_diff > 0:
        # Get days of month we are currently looking at
        iter_month, iter_month_days = monthrange(iter_date.year, iter_date.month)
        # Calculate how many days are left to end of this month
        days_to_month_end = min((iter_month_days - (iter_date.day - 1)), date_diff)
        # Add percentage of the month these days cover to return variable
        delta += days_to_month_end / iter_month_days
        # Reduce leftover days by the amount we already processed
        date_diff -= days_to_month_end

        # Set iteration date to first of next month
        iter_date += relativedelta(months=1, day=1)

    # Return data
    return delta


def get_first_and_last_of_month(date_object: Optional[datetime.date] = None) -> tuple[datetime.date, datetime.date]:
    """
    Returns first and last day of a month as date objects.
    Will either return first/last of current month (if no datetime_object is passed), or will determine first/last of
    the month of the passed datetime.date object
    """
    first_of_month = timezone.now().replace(day=1).date() if not date_object else date_object.replace(day=1)

    # Add one month to first_of_month
    first_of_next_month = first_of_month + relativedelta(months=1)

    # Subtract one day, to get the last of month
    last_of_month = first_of_next_month - datetime.timedelta(days=1)

    return first_of_month, last_of_month


def get_current_year() -> int:
    """
    Returns the current year.
    Useful in model defaults where you want to put a callable.
    """
    return tz_today().year


def check_date_is_weekend(compare_date: datetime.date, weekend_days=(calendar.SATURDAY, calendar.SUNDAY)) -> bool:
    """
    Determines if a given "compare_date" is on a weekday, based on "weekend_days",
    which default to European weekend days.
    """
    return compare_date.weekday() in weekend_days



# Generated by Django 5.0.7 on 2024-09-29 06:46

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("testapp", "0001_initial"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="ModelWithoutRelatedNameOnFieldAndMeta",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "relation_field",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ModelWithoutRelatedNameOnFieldButWithMeta",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "relation_field",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "default_related_name": "model_without_related_name_on_field_but_with_metas",
            },
        ),
    ]



import warnings
from unittest import mock

from django.core import checks
from django.test import SimpleTestCase, override_settings

from ambient_toolbox.system_checks.model_field_name_conventions import check_model_time_based_fields
from testapp.models import ModelNameTimeBasedFieldTest


class CheckModelTimeBasedFieldsTest(SimpleTestCase):
    def test_check_regular(self):
        # Create expected warnings
        datetime_warning = checks.Warning(
            "DateTimeField 'ModelNameTimeBasedFieldTest.wrongly_named_datetime_field' doesn't end with: _at.",
            obj=ModelNameTimeBasedFieldTest.wrongly_named_datetime_field.field,
            id="ambient_toolbox.W001",
        )
        date_warning = checks.Warning(
            "DateField 'ModelNameTimeBasedFieldTest.wrongly_named_date_field' doesn't end with: _date.",
            obj=ModelNameTimeBasedFieldTest.wrongly_named_date_field.field,
            id="ambient_toolbox.W002",
        )

        # Call system check
        error_list = check_model_time_based_fields()

        # Assert warngins
        self.assertEqual(len(error_list), 2)
        self.assertIn(datetime_warning, error_list)
        self.assertIn(date_warning, error_list)

    @override_settings(ALLOWED_MODEL_DATETIME_FIELD_ENDINGS=["wrongly_named_datetime_field", "_at"])
    def test_check_allowlist_works_datetime_field(self):
        # Call system check
        error_list = check_model_time_based_fields()

        # Assert warngins
        self.assertEqual(len(error_list), 1)
        self.assertEqual(error_list[0].id, "ambient_toolbox.W002")

    @override_settings(ALLOWED_MODEL_DATE_FIELD_ENDINGS=["wrongly_named_date_field", "_date"])
    def test_check_allowlist_works_date_field(self):
        # Call system check
        error_list = check_model_time_based_fields()

        # Assert warngins
        self.assertEqual(len(error_list), 1)
        self.assertEqual(error_list[0].id, "ambient_toolbox.W001")

    @override_settings(LOCAL_APPS=["testapp"])
    def test_check_local_apps_are_detected(self):
        # Call system check
        error_list = check_model_time_based_fields()

        # Assert warngins
        self.assertEqual(len(error_list), 2)
        self.assertEqual(error_list[0].id, "ambient_toolbox.W002")
        self.assertEqual(error_list[1].id, "ambient_toolbox.W001")

    @override_settings(INSTALLED_APPS="")
    def test_check_no_local_apps_found(self):
        # Call system check
        with mock.patch.object(warnings, "warn") as mocked_warning:
            error_list = check_model_time_based_fields()

        # Assert warngins
        self.assertEqual(len(error_list), 0)
        mocked_warning.assert_called_once_with(
            "No local apps detected. Therefore, no model fields will be checked for date-time-conventions.",
            Warning,
            stacklevel=2,
        )



import warnings

from django.apps import apps
from django.conf import settings
from django.core import checks
from django.db.models import ForeignKey, ManyToManyField, OneToOneField


def check_model_related_names_for_related_name(*args, **kwargs):
    """
    Checks all model relation fields ('ForeignKey', 'ManyToManyField', "OneToOneField") for having a defined
    "related_name". Either directly via the field or otherwise via the model meta-option "default_related_name".
    """

    if getattr(settings, "LOCAL_APPS", None):
        project_apps = [app.split(".")[-1] for app in settings.LOCAL_APPS]
    # Otherwise, we try some magic...
    else:
        project_apps = [
            app.split(".")[-1] for app in settings.INSTALLED_APPS if app.startswith(settings.ROOT_URLCONF.split(".")[0])
        ]

    # If we still don't get any apps, inform the user
    if len(project_apps) == 0:
        warnings.warn(
            "No local apps detected. Therefore, no model fields will be checked for missing related names.",
            Warning,
            stacklevel=2,
        )

    issue_list = []

    # Iterate all registered models...
    for model in apps.get_models():
        # Check if the model is from your project...
        if model._meta.app_label in project_apps:
            # If the model has a related name, this will be inherited to all relation fields and we're OK
            if model._meta.default_related_name:
                continue

            # Iterate over all fields...
            for field in model._meta.get_fields():
                # Check relation field types...
                if isinstance(field, (ForeignKey, ManyToManyField, OneToOneField)):
                    # Check if the field has a related name set...
                    if not field._related_name:
                        issue_list.append(
                            checks.Warning(
                                f"'{model.__name__}.{field.name}' doesn't have a related name set and neither does the "
                                "model define a default related name.",
                                obj=field,
                                id="ambient_toolbox.W003",
                            )
                        )

    return issue_list
